@@ -2,7 +2,7 @@ use std::collections::HashMap;
 
 use crate::codec::Value;
 
-use super::{MenuHandle, Point, Rect, Size};
+use super::{HotKeyHandle, MenuHandle, Point, Rect, Size};
 
 #[derive(serde::Serialize, serde::Deserialize, Debug, Clone)]
 #[serde(rename_all = "camelCase")]
@@ -215,6 +215,7 @@ pub struct WindowStyle {
     pub can_minimize: bool,
     pub can_maximize: bool,
     pub can_full_screen: bool,
+    pub traffic_light_offset: Option<Point>,
 }
 
 //
@@ -331,6 +332,7 @@ pub struct Key {
     pub logical_shift: Option<i64>,
     pub logical_alt: Option<i64>,
     pub logical_alt_shift: Option<i64>,
+    pub logical_meta: Option<i64>,
 }
 
 #[derive(serde::Serialize, Debug, Clone)]
@@ -338,3 +340,22 @@ pub struct Key {
 pub struct KeyboardMap {
     pub keys: Vec<Key>,
 }
+
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct HotKeyCreateRequest {
+    pub accelerator: Accelerator,
+    pub platform_key: i64,
+}
+
+#[derive(serde::Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct HotKeyDestroyRequest {
+    pub handle: HotKeyHandle,
+}
+
+#[derive(serde::Serialize)]
+#[serde(rename_all = "camelCase")]
+pub struct HotKeyPressed {
+    pub handle: HotKeyHandle,
+}
