@@ -4,8 +4,10 @@ mod bundle;
 mod context;
 mod engine;
 mod engine_manager;
+mod event_channel;
 mod geometry;
 mod handle;
+mod hot_key_manager;
 mod keyboard_map_manager;
 mod macros;
 mod menu_manager;
@@ -22,8 +24,10 @@ pub use bundle::*;
 pub use context::*;
 pub use engine::*;
 pub use engine_manager::*;
+pub use event_channel::*;
 pub use geometry::*;
 pub use handle::*;
+pub use hot_key_manager::*;
 pub use keyboard_map_manager::*;
 pub use macros::*;
 pub use menu_manager::*;
