@@ -1,5 +1,6 @@
 extern crate gtk;
 
+pub mod app_delegate;
 pub mod binary_messenger;
 pub mod drag_context;
 pub mod drag_data;
@@ -7,8 +8,8 @@ pub mod engine;
 pub mod error;
 pub mod flutter;
 pub mod flutter_sys;
+pub mod hot_key;
 pub mod init;
-pub mod key_event;
 pub mod keyboard_map;
 pub mod menu;
 pub mod menu_item;
