@@ -4,6 +4,7 @@ use std::{
     ffi::c_void,
     fmt::Write,
     hash::Hash,
+    mem::ManuallyDrop,
     rc::{Rc, Weak},
 };
 
@@ -20,10 +21,9 @@ use objc::{
 };
 
 use crate::{
-    shell::api_model::{Menu, MenuItem, MenuItemRole},
     shell::{
-        api_model::{Accelerator, CheckStatus, MenuRole},
-        Context, Handle, MenuHandle, MenuManager,
+        api_model::{Accelerator, CheckStatus, Menu, MenuItem, MenuItemRole, MenuRole},
+        Context, Handle, MenuDelegate, MenuHandle, MenuManager,
     },
     util::{update_diff, DiffResult, LateRefCell},
 };
@@ -51,6 +51,7 @@ impl Hash for StrongPtrWrapper {
 
 pub struct PlatformMenuManager {
     context: Context,
+    weak_self: LateRefCell<Weak<PlatformMenuManager>>,
     app_menu: RefCell<Option<Rc<PlatformMenu>>>,
     window_menus: RefCell<HashMap<StrongPtrWrapper, Rc<PlatformMenu>>>,
     update_handle: RefCell<Option<Handle>>,
@@ -60,12 +61,17 @@ impl PlatformMenuManager {
     pub fn new(context: Context) -> Self {
         Self {
             context,
+            weak_self: LateRefCell::new(),
             app_menu: RefCell::new(None),
             window_menus: RefCell::new(HashMap::new()),
             update_handle: RefCell::new(None),
         }
     }
 
+    pub(crate) fn assign_weak_self(&self, weak_self: Weak<PlatformMenuManager>) {
+        self.weak_self.set(weak_self);
+    }
+
     fn update_menu(&self) {
         unsafe {
             let mut menu = self.app_menu.borrow().clone();
@@ -95,15 +101,11 @@ impl PlatformMenuManager {
     }
 
     fn schedule_update(&self) {
-        let context_clone = self.context.clone();
+        let weak_self = self.weak_self.borrow().clone();
         if let Some(context) = self.context.get() {
             let callback = context.run_loop.borrow().schedule_now(move || {
-                if let Some(context) = context_clone.get() {
-                    context
-                        .menu_manager
-                        .borrow()
-                        .get_platform_menu_manager()
-                        .update_menu();
+                if let Some(s) = weak_self.upgrade() {
+                    s.update_menu();
                 }
             });
             self.update_handle.borrow_mut().replace(callback);
@@ -155,19 +157,23 @@ impl PlatformMenuManager {
 }
 
 pub struct PlatformMenu {
-    context: Context,
     handle: MenuHandle,
     pub(super) menu: StrongPtr,
     previous_menu: RefCell<Menu>,
     id_to_menu_item: RefCell<HashMap<i64, StrongPtr>>,
     target: StrongPtr,
     weak_self: LateRefCell<Weak<PlatformMenu>>,
+    delegate: Weak<RefCell<dyn MenuDelegate>>,
 }
 
 const ITEM_TAG: NSInteger = 9999;
 
 impl PlatformMenu {
-    pub fn new(context: Context, handle: MenuHandle) -> Self {
+    pub fn new(
+        _context: Context,
+        handle: MenuHandle,
+        delegate: Weak<RefCell<dyn MenuDelegate>>,
+    ) -> Self {
         unsafe {
             let menu: id = NSMenu::alloc(nil).initWithTitle_(*to_nsstring(""));
             let () = msg_send![menu, setAutoenablesItems: NO];
@@ -178,13 +184,13 @@ impl PlatformMenu {
             let () = msg_send![menu, setDelegate:*target];
 
             Self {
-                context,
                 handle,
                 menu: StrongPtr::new(menu),
                 previous_menu: RefCell::new(Default::default()),
                 id_to_menu_item: RefCell::new(HashMap::new()),
                 target,
                 weak_self: LateRefCell::new(),
+                delegate,
             }
         }
     }
@@ -192,7 +198,7 @@ impl PlatformMenu {
     pub fn assign_weak_self(&self, weak: Weak<PlatformMenu>) {
         self.weak_self.set(weak.clone());
         unsafe {
-            let state_ptr = Box::into_raw(Box::new(weak)) as *mut c_void;
+            let state_ptr = weak.into_raw() as *mut c_void;
             (**self.target).set_ivar("imState", state_ptr);
         }
     }
@@ -298,9 +304,9 @@ impl PlatformMenu {
             .filter_map(|f| f.submenu)
             .collect();
 
-        if let Some(context) = self.context.get() {
+        if let Some(delegate) = self.delegate.upgrade() {
             for c in children {
-                let menu = context.menu_manager.borrow().get_platform_menu(c);
+                let menu = delegate.borrow().get_platform_menu(c);
                 if let Ok(menu) = menu {
                     menu.prepare_for_app_menu();
                 }
@@ -413,10 +419,10 @@ impl PlatformMenu {
             "space" => 0x0020,
             "tab" => 0x0009,
             "enter" => 0x000d,
-            "up arrow" => 0xF700,
-            "down arrow" => 0xF701,
-            "left arrow" => 0xF702,
-            "right arrow" => 0xF703,
+            "arrow up" => 0xF700,
+            "arrow down" => 0xF701,
+            "arrow left" => 0xF702,
+            "arrow right" => 0xF703,
             _ => label.chars().next().unwrap_or(0 as char) as u32,
         };
         let mut res = String::new();
@@ -494,21 +500,18 @@ impl PlatformMenu {
     }
 
     fn menu_item_action(&self, item: id) {
-        if let Some(context) = self.context.get() {
+        if let Some(delegate) = self.delegate.upgrade() {
             let item_id = unsafe {
                 let object: id = msg_send![item, representedObject];
                 msg_send![object, longLongValue]
             };
-            context
-                .menu_manager
-                .borrow()
-                .on_menu_action(self.handle, item_id);
+            delegate.borrow().on_menu_action(self.handle, item_id);
         }
     }
 
     fn on_menu_will_open(&self) {
-        if let Some(context) = self.context.get() {
-            context.menu_manager.borrow().on_menu_open(self.handle);
+        if let Some(delegate) = self.delegate.upgrade() {
+            delegate.borrow().on_menu_open(self.handle);
         }
     }
 
@@ -557,12 +560,12 @@ lazy_static! {
 }
 
 extern "C" fn dealloc(this: &Object, _sel: Sel) {
-    let state_ptr = unsafe {
-        let state_ptr: *mut c_void = *this.get_ivar("imState");
-        &mut *(state_ptr as *mut Weak<PlatformMenu>)
-    };
     unsafe {
-        Box::from_raw(state_ptr);
+        let state_ptr = {
+            let state_ptr: *mut c_void = *this.get_ivar("imState");
+            state_ptr as *const PlatformMenu
+        };
+        Weak::from_raw(state_ptr);
 
         let superclass = superclass(this);
         let () = msg_send![super(this, superclass), dealloc];
@@ -570,22 +573,28 @@ extern "C" fn dealloc(this: &Object, _sel: Sel) {
 }
 
 extern "C" fn on_action(this: &Object, _sel: Sel, sender: id) {
-    let state_ptr = unsafe {
-        let state_ptr: *mut c_void = *this.get_ivar("imState");
-        &mut *(state_ptr as *mut Weak<PlatformMenu>)
+    let state = unsafe {
+        let state_ptr = {
+            let state_ptr: *mut c_void = *this.get_ivar("imState");
+            state_ptr as *const PlatformMenu
+        };
+        ManuallyDrop::new(Weak::from_raw(state_ptr))
     };
-    let upgraded = state_ptr.upgrade();
+    let upgraded = state.upgrade();
     if let Some(upgraded) = upgraded {
         upgraded.menu_item_action(sender);
     }
 }
 
 extern "C" fn menu_will_open(this: &Object, _: Sel, _menu: id) {
-    let state_ptr = unsafe {
-        let state_ptr: *mut c_void = *this.get_ivar("imState");
-        &mut *(state_ptr as *mut Weak<PlatformMenu>)
+    let state = unsafe {
+        let state_ptr = {
+            let state_ptr: *mut c_void = *this.get_ivar("imState");
+            state_ptr as *const PlatformMenu
+        };
+        ManuallyDrop::new(Weak::from_raw(state_ptr))
     };
-    let upgraded = state_ptr.upgrade();
+    let upgraded = state.upgrade();
     if let Some(upgraded) = upgraded {
         upgraded.on_menu_will_open();
     }
