@@ -2,34 +2,35 @@ use std::{
     cell::{Cell, RefCell},
     collections::HashMap,
     ffi::c_void,
+    mem::ManuallyDrop,
     rc::{Rc, Weak},
     time::Duration,
 };
 
 use cocoa::{
     appkit::{
-        NSEvent, NSEventType, NSView, NSWindow, NSWindowCollectionBehavior, NSWindowStyleMask,
+        CGPoint, NSApplication, NSEvent, NSEventType, NSScreen, NSView, NSViewHeightSizable,
+        NSViewWidthSizable, NSWindow, NSWindowCollectionBehavior, NSWindowStyleMask,
+        NSWindowTabbingMode,
     },
     base::{id, nil, BOOL, NO, YES},
-    foundation::{NSArray, NSInteger, NSPoint, NSRect, NSSize, NSString, NSUInteger},
-};
-use cocoa::{
-    appkit::{NSScreen, NSWindowTabbingMode},
-    foundation::NSProcessInfo,
+    foundation::{
+        NSArray, NSInteger, NSPoint, NSProcessInfo, NSRect, NSSize, NSString, NSUInteger,
+    },
 };
 
+use core_foundation::base::CFRelease;
+use core_graphics::event::CGEventType;
+
 use objc::{
     declare::ClassDecl,
-    rc::{StrongPtr, WeakPtr},
-};
-use objc::{
-    rc::autoreleasepool,
+    rc::{autoreleasepool, StrongPtr, WeakPtr},
     runtime::{Class, Object, Sel},
 };
 
 use NSEventType::{
-    NSLeftMouseDown, NSLeftMouseUp, NSMouseEntered, NSMouseExited, NSMouseMoved, NSRightMouseDown,
-    NSRightMouseUp,
+    NSLeftMouseDown, NSLeftMouseDragged, NSLeftMouseUp, NSMouseEntered, NSMouseExited,
+    NSMouseMoved, NSRightMouseDown, NSRightMouseUp,
 };
 
 use crate::{
@@ -67,6 +68,10 @@ pub struct PlatformWindow {
     drag_context: LateRefCell<DragContext>,
     last_event: RefCell<HashMap<u64, StrongPtr>>,
     ignore_enter_leave_until: Cell<f64>,
+    window_buttons: StrongPtr,
+    flutter_view: LateRefCell<StrongPtr>,
+    mouse_down: Cell<bool>,
+    mouse_dragged: Cell<bool>,
 }
 
 #[link(name = "AppKit", kind = "framework")]
@@ -74,6 +79,10 @@ extern "C" {
     pub static NSPasteboardTypeFileURL: id;
 }
 
+extern "C" {
+    fn im_link_objc_dummy_method();
+}
+
 impl PlatformWindow {
     pub fn new(
         context: Context,
@@ -104,6 +113,9 @@ impl PlatformWindow {
 
             window.setDelegate_(*platform_delegate);
 
+            let window_buttons: id = msg_send![class!(IMWindowButtons), new];
+            let window_buttons = StrongPtr::new(window_buttons);
+
             Self {
                 context,
                 platform_window: window,
@@ -117,6 +129,10 @@ impl PlatformWindow {
                 last_event: RefCell::new(HashMap::new()),
                 drag_context: LateRefCell::new(),
                 ignore_enter_leave_until: Cell::new(0.0),
+                window_buttons,
+                flutter_view: LateRefCell::new(),
+                mouse_down: Cell::new(false),
+                mouse_dragged: Cell::new(false),
             }
         })
     }
@@ -125,19 +141,37 @@ impl PlatformWindow {
         self.weak_self.set(weak.clone());
 
         unsafe {
-            let state_ptr = Box::into_raw(Box::new(weak.clone())) as *mut c_void;
+            // dummy method to force rust to link macos_extra.a
+            im_link_objc_dummy_method();
+
+            let state_ptr = weak.clone().into_raw() as *mut c_void;
             (**self.platform_delegate).set_ivar("imState", state_ptr);
 
-            let state_ptr = Box::into_raw(Box::new(weak.clone())) as *mut c_void;
+            let state_ptr = weak.clone().into_raw() as *mut c_void;
             (**self.platform_window).set_ivar("imState", state_ptr);
 
-            let () =
-                msg_send![*self.platform_window, setContentViewController: *engine.view_controller];
+            let flutter_view: id = msg_send![*engine.view_controller, view];
+            self.flutter_view.set(StrongPtr::retain(flutter_view));
+
+            let view: id = msg_send![class!(IMContentView), alloc];
+            let view: id = msg_send![view, init];
+
+            let () = msg_send![*self.platform_window, setContentView: view];
+            let () = msg_send![view, addSubview: flutter_view];
+
+            // Add traffic light
+            let () = msg_send![view, addSubview: *self.window_buttons];
 
             let () = msg_send![*engine.view_controller, setMouseTrackingMode: 3]; // always track mouse
 
             // Temporarily set non empty window size so that flutter engine doesn't complain
             NSWindow::setContentSize_(*self.platform_window, Size::wh(1.0, 1.0).into());
+
+            let () = msg_send![flutter_view, setFrame: NSRect::new(
+                NSPoint::new(0.0, 0.0),
+                NSSize::new(1.0, 1.0)
+            )];
+            NSView::setAutoresizingMask_(flutter_view, NSViewWidthSizable | NSViewHeightSizable);
         }
 
         if let Some(context) = self.context.get() {
@@ -362,15 +396,20 @@ impl PlatformWindow {
             let mut mask: NSWindowStyleMask = NSWindowStyleMask::NSBorderlessWindowMask;
 
             if style.frame == WindowFrame::Regular {
-                NSWindow::setMovable_(*self.platform_window, YES);
                 NSWindow::setTitlebarAppearsTransparent_(*self.platform_window, NO);
             } else {
-                NSWindow::setMovable_(*self.platform_window, NO);
                 NSWindow::setTitlebarAppearsTransparent_(*self.platform_window, YES);
             }
 
             if style.frame == WindowFrame::NoTitle {
                 mask |= NSWindowStyleMask::NSFullSizeContentViewWindowMask;
+                let () = msg_send![*self.window_buttons, setEnabled: YES];
+                if let Some(offset) = &style.traffic_light_offset {
+                    let offset: NSPoint = offset.into();
+                    let () = msg_send![*self.window_buttons, setOrigin: offset];
+                }
+            } else {
+                let () = msg_send![*self.window_buttons, setEnabled: NO];
             }
 
             if style.frame != WindowFrame::NoFrame {
@@ -411,6 +450,21 @@ impl PlatformWindow {
         Ok(())
     }
 
+    pub fn save_position_to_string(&self) -> PlatformResult<String> {
+        unsafe {
+            let string: id = msg_send![*self.platform_window, stringWithSavedFrame];
+            Ok(from_nsstring(string))
+        }
+    }
+
+    pub fn restore_position_from_string(&self, position: String) -> PlatformResult<()> {
+        unsafe {
+            let position = to_nsstring(&position);
+            let () = msg_send![*self.platform_window, setFrameFromString:*position];
+        }
+        Ok(())
+    }
+
     pub fn is_modal(&self) -> bool {
         self.modal_close_callback.borrow().is_some()
     }
@@ -427,13 +481,14 @@ impl PlatformWindow {
     fn show_when_ready(weak_self: Weak<PlatformWindow>, attempt: i32) {
         if let Some(s) = weak_self.upgrade() {
             autoreleasepool(|| unsafe {
-                let view = NSWindow::contentView(*s.platform_window);
-                let subviews: id = msg_send![view, subviews];
+                let view = s.flutter_view.borrow().clone();
+
+                let subviews: id = msg_send![*view, subviews];
                 let view = {
                     if subviews.count() > 0 {
                         subviews.objectAtIndex(0)
                     } else {
-                        view
+                        *view
                     }
                 };
 
@@ -453,7 +508,7 @@ impl PlatformWindow {
                             panic!("Expected IOSurface content");
                         }
                         let scale = NSWindow::backingScaleFactor(*s.platform_window);
-                        let content_size = NSView::frame(NSWindow::contentView(*s.platform_window));
+                        let content_size = NSView::frame(*s.flutter_view.borrow().clone());
 
                         let expected_width = scale * content_size.size.width;
                         let expected_height = scale * content_size.size.height;
@@ -475,17 +530,15 @@ impl PlatformWindow {
                     if let Some(delegate) = s.delegate.upgrade() {
                         delegate.visibility_changed(true);
                     };
-                } else {
-                    if let Some(context) = s.context.get() {
-                        // wait until we have content generated (with proper size)
-                        context
-                            .run_loop
-                            .borrow()
-                            .schedule(Duration::from_secs_f64(1.0 / 60.0), move || {
-                                Self::show_when_ready(weak_self, attempt + 1)
-                            })
-                            .detach();
-                    }
+                } else if let Some(context) = s.context.get() {
+                    // wait until we have content generated (with proper size)
+                    context
+                        .run_loop
+                        .borrow()
+                        .schedule(Duration::from_secs_f64(1.0 / 60.0), move || {
+                            Self::show_when_ready(weak_self, attempt + 1)
+                        })
+                        .detach();
                 }
             });
         }
@@ -551,6 +604,15 @@ impl PlatformWindow {
         Ok(())
     }
 
+    pub fn activate(&self) -> PlatformResult<bool> {
+        unsafe {
+            let app = NSApplication::sharedApplication(nil);
+            NSApplication::activateIgnoringOtherApps_(app, YES);
+            NSWindow::makeKeyAndOrderFront_(*self.platform_window, nil);
+        }
+        Ok(true)
+    }
+
     unsafe fn synthetize_mouse_up_event(&self) {
         let last_event = self
             .last_event
@@ -568,23 +630,19 @@ impl PlatformWindow {
 
         if let Some(event) = last_event {
             let opposite = match event.eventType() {
-                NSLeftMouseDown => NSLeftMouseUp,
-                NSRightMouseDown => NSRightMouseUp,
+                NSLeftMouseDown => CGEventType::LeftMouseUp,
+                NSRightMouseDown => CGEventType::RightMouseUp,
                 _ => return,
             };
 
-            let opposite: id = msg_send![class!(NSEvent), mouseEventWithType: opposite
-                location:NSEvent::mouseLocation(*event)
-                modifierFlags:NSEvent::modifierFlags(*event)
-                timestamp:NSEvent::timestamp(*event)
-                windowNumber:NSEvent::windowNumber(*event)
-                context:NSEvent::context(*event)
-                eventNumber:event.eventNumber()
-                clickCount:1
-                pressure:1
-            ];
+            let event = NSEvent::CGEvent(*event) as core_graphics::sys::CGEventRef;
+            let event = CGEventCreateCopy(event);
+            CGEventSetType(event, opposite);
 
-            let () = msg_send![*self.platform_window, sendEvent: opposite];
+            let synthetized: id = msg_send![class!(NSEvent), eventWithCGEvent: event];
+            CFRelease(event as *mut _);
+
+            let () = msg_send![*self.platform_window, sendEvent: synthetized];
         }
     }
 
@@ -633,6 +691,17 @@ impl PlatformWindow {
         });
     }
 
+    pub fn on_layout(&self) {
+        if unsafe { NSWindow::inLiveResize(*self.platform_window) } == YES {
+            if let Some(context) = self.context.get() {
+                // Neither run loop nor main dispatch queue are running during
+                // window resizing; So we poll the run loop manually to keep things
+                // updated.
+                context.run_loop.borrow().platform_run_loop.poll();
+            }
+        }
+    }
+
     pub fn should_send_event(&self, event: StrongPtr) -> bool {
         let event_type = unsafe { NSEvent::eventType(*event) };
         if event_type == NSMouseEntered || event_type == NSMouseExited {
@@ -646,9 +715,95 @@ impl PlatformWindow {
                 return false;
             }
         }
+        self.check_window_dragging(event);
         true
     }
 
+    // Special handling for dragging window with popup menu open.
+    //
+    // If user drags mouse on window with popup menu open, the LeftMouseDown event
+    // swallowed (used to close the popup menu) and the window start getting
+    // NSLeftMouseDragged events. We try to detect that, and upon getting the second
+    // NSLeftMouseDragged event without prior NSLeftMouseDown, we synthetize the
+    // NSLeftMouseDown event ourself (necessary for flutter view to start getting
+    // mouse drag events). We ignore the first NSLeftMouseDragged event because
+    // that's sometimes posted without a subsequent NSLeftMouseUp event.
+    //
+    // This all is necessary to have custom titlebars draggable while popup menu
+    // is open.
+    fn check_window_dragging(&self, event: StrongPtr) {
+        let event_type = unsafe { NSEvent::eventType(*event) };
+
+        // println!(
+        //     "EVent type {:?} {} {}",
+        //     event_type,
+        //     self.mouse_down.get(),
+        //     self.mouse_dragged.get()
+        // );
+
+        // NSMouseMoved after NSLeftMouseDragged without NSLeftMouseUp
+        if event_type == NSMouseMoved {
+            if self.mouse_down.get() {
+                unsafe {
+                    // println!("Synthetizing up");
+                    self.synthetize_mouse_up_event();
+                }
+            }
+            self.mouse_down.set(false);
+            self.mouse_dragged.set(false);
+        }
+
+        if event_type == NSLeftMouseDown {
+            self.mouse_down.set(true);
+            self.mouse_dragged.set(false);
+        } else if event_type == NSLeftMouseUp {
+            self.mouse_down.set(false);
+            self.mouse_dragged.set(false);
+        } else if event_type == NSLeftMouseDragged
+            && !self.mouse_down.get()
+            && !self.mouse_dragged.get()
+        {
+            // first NSLeftMouseDragged, ignore it because sometimes window
+            // server sends it without subsequent NSLeftMouseUp
+            self.mouse_dragged.set(true);
+        } else if event_type == NSLeftMouseDragged
+            && !self.mouse_down.get()
+            && self.mouse_dragged.get()
+        {
+            // Second NSLeftMouseDragged without prior NSLeftMouseDown; This likely
+            // means user started dragging window while popup menu was opened.
+            // On this case we synthetize LeftMouseDown event at the location
+            // of lastest hitTest in IMContentView. While window server swallows
+            // the mouseDown event, it still generates hitTest at the location
+            // where user pressed the button to possibly initiate regular
+            // window drag.
+            unsafe {
+                let event = NSEvent::CGEvent(*event) as core_graphics::sys::CGEventRef;
+                let event = CGEventCreateCopy(event);
+                CGEventSetType(event, CGEventType::LeftMouseDown);
+                let location: CGPoint = msg_send![
+                    NSWindow::contentView(*self.platform_window),
+                    lastHitTestScreen
+                ];
+                let location_win: CGPoint = msg_send![
+                    NSWindow::contentView(*self.platform_window),
+                    lastHitTestWindow
+                ];
+                CGEventSetLocation(event, location);
+                CGEventSetWindowLocation(event, location_win);
+
+                println!("Synthetizing left mouse down");
+
+                let synthetized: id = msg_send![class!(NSEvent), eventWithCGEvent: event];
+                CFRelease(event as *mut _);
+
+                let () = msg_send![*self.platform_window, sendEvent: synthetized];
+                self.mouse_down.set(true);
+                self.mouse_dragged.set(false);
+            }
+        }
+    }
+
     pub fn set_pending_effect(&self, effect: DragEffect) {
         self.drag_context.borrow_mut().set_pending_effect(effect);
     }
@@ -744,6 +899,7 @@ impl PlatformWindow {
             context
                 .menu_manager
                 .borrow()
+                .borrow()
                 .get_platform_menu_manager()
                 .set_menu_for_window(self.platform_window.clone(), menu);
         }
@@ -766,10 +922,6 @@ unsafe impl Sync for WindowClass {}
 struct WindowDelegateClass(*const Class);
 unsafe impl Sync for WindowDelegateClass {}
 
-extern "C" fn accepts_first_mouse(_this: &Object, _sel: Sel, _event: id) -> BOOL {
-    YES
-}
-
 lazy_static! {
     static ref WINDOW_CLASS: WindowClass = unsafe {
         // FlutterView doesn't override acceptsFirstMouse: so we do it here
@@ -785,11 +937,19 @@ lazy_static! {
         let mut decl = ClassDecl::new("IMFlutterWindow", window_superclass).unwrap();
 
         decl.add_method(sel!(dealloc), dealloc as extern "C" fn(&Object, Sel));
+
+        decl.add_method(sel!(layoutIfNeeded), layout_if_needed as extern "C" fn(&mut Object, Sel));
+
         decl.add_method(
             sel!(sendEvent:),
             send_event as extern "C" fn(&mut Object, Sel, id),
         );
 
+        decl.add_method(
+            sel!(canBecomeKeyWindow),
+            can_become_key_window as extern "C" fn(&Object, Sel) -> BOOL,
+        );
+
         decl.add_method(
             sel!(draggingEntered:),
             dragging_entered as extern "C" fn(&mut Object, Sel, id) -> NSDragOperation,
@@ -867,11 +1027,14 @@ fn with_state<F>(this: &Object, callback: F)
 where
     F: FnOnce(Rc<PlatformWindow>),
 {
-    let state_ptr = unsafe {
-        let state_ptr: *mut c_void = *this.get_ivar("imState");
-        &mut *(state_ptr as *mut Weak<PlatformWindow>)
+    let state = unsafe {
+        let state_ptr = {
+            let state_ptr: *mut c_void = *this.get_ivar("imState");
+            state_ptr as *const PlatformWindow
+        };
+        ManuallyDrop::new(Weak::from_raw(state_ptr))
     };
-    let upgraded = state_ptr.upgrade();
+    let upgraded = state.upgrade();
     if let Some(upgraded) = upgraded {
         callback(upgraded);
     }
@@ -882,11 +1045,14 @@ where
     F: FnOnce(Rc<PlatformWindow>) -> R,
     FR: FnOnce() -> R,
 {
-    let state_ptr = unsafe {
-        let state_ptr: *mut c_void = *this.get_ivar("imState");
-        &mut *(state_ptr as *mut Weak<PlatformWindow>)
+    let state = unsafe {
+        let state_ptr = {
+            let state_ptr: *mut c_void = *this.get_ivar("imState");
+            state_ptr as *const PlatformWindow
+        };
+        ManuallyDrop::new(Weak::from_raw(state_ptr))
     };
-    let upgraded = state_ptr.upgrade();
+    let upgraded = state.upgrade();
     if let Some(upgraded) = upgraded {
         callback(upgraded)
     } else {
@@ -906,6 +1072,14 @@ where
     });
 }
 
+// #[link(name = "CoreGraphics", kind = "framework")]
+extern "C" {
+    fn CGEventSetType(event: core_graphics::sys::CGEventRef, eventType: CGEventType);
+    fn CGEventSetLocation(event: core_graphics::sys::CGEventRef, location: CGPoint);
+    fn CGEventSetWindowLocation(event: core_graphics::sys::CGEventRef, location: CGPoint);
+    fn CGEventCreateCopy(event: core_graphics::sys::CGEventRef) -> core_graphics::sys::CGEventRef;
+}
+
 extern "C" fn window_did_move(this: &Object, _: Sel, _: id) {
     with_state_delegate(this, |_state, _delegate| {});
 }
@@ -930,6 +1104,7 @@ extern "C" fn window_will_close(this: &Object, _: Sel, _: id) {
             context
                 .menu_manager
                 .borrow()
+                .borrow()
                 .get_platform_menu_manager()
                 .window_will_close(state.platform_window.clone());
         }
@@ -937,12 +1112,17 @@ extern "C" fn window_will_close(this: &Object, _: Sel, _: id) {
     });
 }
 
+extern "C" fn accepts_first_mouse(_this: &Object, _sel: Sel, _event: id) -> BOOL {
+    YES
+}
+
 extern "C" fn window_did_become_key(this: &Object, _: Sel, _: id) {
     with_state_delegate(this, |state, _delegate| {
         if let Some(context) = state.context.get() {
             context
                 .menu_manager
                 .borrow()
+                .borrow()
                 .get_platform_menu_manager()
                 .window_did_become_active(state.platform_window.clone());
         }
@@ -955,12 +1135,28 @@ extern "C" fn window_did_resign_key(this: &Object, _: Sel, _: id) {
             context
                 .menu_manager
                 .borrow()
+                .borrow()
                 .get_platform_menu_manager()
                 .window_did_resign_active(state.platform_window.clone());
         }
     });
 }
 
+extern "C" fn layout_if_needed(this: &mut Object, _sel: Sel) {
+    unsafe {
+        with_state(this, move |state| {
+            state.on_layout();
+        });
+        let superclass = superclass(this);
+        let () = msg_send![super(this, superclass), layoutIfNeeded];
+    }
+}
+
+extern "C" fn can_become_key_window(_this: &Object, _: Sel) -> BOOL {
+    // needed for frameless windows to accept keyboard input.
+    YES
+}
+
 extern "C" fn send_event(this: &mut Object, _: Sel, e: id) {
     unsafe {
         let event = StrongPtr::retain(e);
@@ -1047,12 +1243,12 @@ extern "C" fn dragging_session_ended_at_point(
 }
 
 extern "C" fn dealloc(this: &Object, _sel: Sel) {
-    let state_ptr = unsafe {
-        let state_ptr: *mut c_void = *this.get_ivar("imState");
-        &mut *(state_ptr as *mut Weak<PlatformWindow>)
-    };
     unsafe {
-        Box::from_raw(state_ptr);
+        let state_ptr = {
+            let state_ptr: *mut c_void = *this.get_ivar("imState");
+            state_ptr as *const PlatformWindow
+        };
+        Weak::from_raw(state_ptr);
 
         let superclass = superclass(this);
         let () = msg_send![super(this, superclass), dealloc];
