@@ -1,12 +1,16 @@
 use super::{binary_messenger::PlatformBinaryMessenger, error::PlatformResult};
 
-pub struct PlatformEngine {}
+pub type PlatformEngineType = isize;
+
+pub struct PlatformEngine {
+    pub(crate) handle: PlatformEngineType,
+}
 
 pub type PlatformPlugin = isize;
 
 impl PlatformEngine {
     pub fn new(_plugins: &[PlatformPlugin]) -> Self {
-        PlatformEngine {}
+        PlatformEngine { handle: 0 }
     }
 
     pub fn new_binary_messenger(&self) -> PlatformBinaryMessenger {
