@@ -6,14 +6,14 @@ use std::{
     slice,
 };
 
-use windows::{create_instance, IUnknown, Interface, HRESULT};
+use windows::{IUnknown, Interface, HRESULT};
 
 use crate::util::OkLog;
 
 use super::{
     all_bindings::*,
     drag_util::{CLSID_DragDropHelper, DataUtil},
-    util::{com_object_from_ptr, get_raw_ptr},
+    util::{com_object_from_ptr, create_instance, get_raw_ptr},
 };
 
 pub trait DropTargetDelegate {
@@ -318,7 +318,7 @@ impl EnumFORMATETC {
             self.index += 1;
             offset += 1;
         }
-        if pcelt_fetched != std::ptr::null_mut() {
+        if !pcelt_fetched.is_null() {
             unsafe {
                 *pcelt_fetched = offset as u32;
             }
@@ -585,7 +585,7 @@ impl DataObject {
         _pformatect_in: *mut FORMATETC,
         _pformatetc_out: *mut FORMATETC,
     ) -> ::windows::HRESULT {
-        HRESULT(E_NOTIMPL as u32)
+        E_NOTIMPL
     }
 
     fn set_data(
@@ -627,13 +627,13 @@ impl DataObject {
                         if let Some(stream) = stream {
                             loop {
                                 let mut num_read: u32 = 0;
-                                if !stream
+                                if stream
                                     .Read(
                                         buf.as_mut_ptr() as *mut _,
                                         buf.len() as u32,
                                         &mut num_read as *mut _,
                                     )
-                                    .is_ok()
+                                    .is_err()
                                 {
                                     break;
                                 }
@@ -693,18 +693,18 @@ impl DataObject {
         _p_adv_sink: ::std::option::Option<IAdviseSink>,
         _pdw_connection: *mut u32,
     ) -> ::windows::HRESULT {
-        HRESULT(E_NOTIMPL as u32)
+        E_NOTIMPL
     }
 
     fn d_unadvise(&self, _dw_connection: u32) -> ::windows::HRESULT {
-        HRESULT(E_NOTIMPL as u32)
+        E_NOTIMPL
     }
 
     fn enum_d_advise(
         &self,
         _ppenum_advise: *mut ::std::option::Option<IEnumSTATDATA>,
     ) -> ::windows::HRESULT {
-        HRESULT(E_NOTIMPL as u32)
+        E_NOTIMPL
     }
 
     unsafe extern "system" fn _query_interface(
