@@ -20,7 +20,8 @@ use super::{
         WindowGeometryRequest, WindowStyle,
     },
     platform::window::PlatformWindow,
-    Context, EngineHandle, WindowMethodCallReply, WindowMethodCallResult, WindowMethodInvoker,
+    Context, EngineHandle, MenuDelegate, WindowMethodCallReply, WindowMethodCallResult,
+    WindowMethodInvoker,
 };
 
 #[derive(serde::Serialize, serde::Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq)]
@@ -131,6 +132,10 @@ impl Window {
         self.platform_window().hide().map_err(|e| e.into())
     }
 
+    fn activate(&self) -> Result<bool> {
+        self.platform_window().activate().map_err(|e| e.into())
+    }
+
     fn set_geometry(&self, geometry: WindowGeometryRequest) -> Result<WindowGeometryFlags> {
         self.platform_window()
             .set_geometry(geometry)
@@ -159,6 +164,18 @@ impl Window {
             .map_err(|e| e.into())
     }
 
+    fn save_position_to_string(&self) -> Result<String> {
+        self.platform_window()
+            .save_position_to_string()
+            .map_err(|e| e.into())
+    }
+
+    fn restore_position_from_string(&self, position: String) -> Result<()> {
+        self.platform_window()
+            .restore_position_from_string(position)
+            .map_err(|e| e.into())
+    }
+
     fn perform_window_drag(&self) -> Result<()> {
         self.platform_window()
             .perform_window_drag()
@@ -179,6 +196,7 @@ impl Window {
             let menu = context
                 .menu_manager
                 .borrow()
+                .borrow()
                 .get_platform_menu(request.handle);
             match menu {
                 Ok(menu) => self
@@ -194,6 +212,7 @@ impl Window {
             let menu = context
                 .menu_manager
                 .borrow()
+                .borrow()
                 .get_platform_menu(request.handle)?;
             self.platform_window()
                 .hide_popup_menu(menu)
@@ -213,7 +232,11 @@ impl Window {
         if let Some(context) = self.context.get() {
             match request.handle {
                 Some(handle) => {
-                    let menu = context.menu_manager.borrow().get_platform_menu(handle)?;
+                    let menu = context
+                        .menu_manager
+                        .borrow()
+                        .borrow()
+                        .get_platform_menu(handle)?;
                     self.platform_window()
                         .set_window_menu(Some(menu))
                         .map_err(|e| e.into())
@@ -276,6 +299,9 @@ impl Window {
             method::window::HIDE => {
                 return Self::reply(reply, &arg, |()| self.hide());
             }
+            method::window::ACTIVATE => {
+                return Self::reply(reply, &arg, |()| self.activate());
+            }
             method::window::SET_GEOMETRY => {
                 return Self::reply(reply, &arg, |geometry| self.set_geometry(geometry));
             }
@@ -291,6 +317,14 @@ impl Window {
             method::window::SET_TITLE => {
                 return Self::reply(reply, &arg, |title| self.set_title(title));
             }
+            method::window::SAVE_POSITION_TO_STRING => {
+                return Self::reply(reply, &arg, |()| self.save_position_to_string());
+            }
+            method::window::RESTORE_POSITION_FROM_STRING => {
+                return Self::reply(reply, &arg, |position: String| {
+                    self.restore_position_from_string(position)
+                });
+            }
             method::window::PERFORM_WINDOW_DRAG => {
                 return Self::reply(reply, &arg, |()| self.perform_window_drag());
             }
