@@ -183,7 +183,7 @@ impl WindowManager {
         parent: WindowHandle,
     ) -> WindowMethodCallResult {
         self.create_window(argument, Some(parent))
-            .map_err(|e| MethodCallError::from(e))
+            .map_err(MethodCallError::from)
             .map(|win| to_value(&WindowCreateResponse { window_handle: win }).unwrap())
     }
 
@@ -196,7 +196,7 @@ impl WindowManager {
             let manager = context.message_manager.borrow();
             self.windows
                 .get(&handle)
-                .and_then(|w| manager.get_message_sender(w.engine_handle, channel_name))
+                .map(|w| manager.get_message_sender(w.engine_handle, channel_name))
         } else {
             None
         }
