@@ -4,7 +4,6 @@ mod diff;
 pub mod errno;
 mod log;
 
-pub use self::diff::*;
-pub use self::log::*;
+pub use self::{diff::*, log::*};
 pub use capsule::*;
 pub use cell::*;
